@@ -38,133 +38,56 @@ def main(input_file):
     R = kB * NA
     # The default volume is 1 dm3 (=1e-3 m3), this can be changed
     # But concentrations should be adjusted accordingly.
-    V = 1e-3  # Volume in m3 (default=1e-3 m3)
 
     # The function read_input(name_of_the_input_file) reads the input data
     # The input file should be in the same folder with the solver
-    (
-        T,
-        Nspecies,
-        x0,
-        names,
-        charges,
-        mu_0,
-        mu_ex,
-        gas_names,
-        muex_gas,
-        cgas,
-        Nreactions,
-        stoichiometry,
-        Nbalances,
-        balances,
-        lnK,
-        compute_Kdes,
-        solvent_index,
-        pure_density,
-        Ngas_species,
-        impose_Ptotal,
-        Ptotal,
-        gas_composition,
-    ) = read_input(input_file)
+    input_parameters = read_input(input_file)
+    input_parameters['V'] = 1e-3  # Volume in m3 (default=1e-3 m3)
 
     # Compute initial amounts for balances
     # Gases are treated differently since the concentration of gas species
     # change in our calculations
-    if impose_Ptotal:
-        init_amounts = np.zeros((Nbalances + Ngas_species, len(Ptotal)))
+    if input_parameters['impose_ptotal']:
+        input_parameters['init_amounts'] = np.zeros((input_parameters['Nbalances'] + input_parameters['Ngas'], len(input_parameters['Ptotal'])))
     else:
-        init_amounts = np.zeros((Nbalances, len(cgas[0, :])))
+        input_parameters['init_amounts'] = np.zeros((input_parameters['Nbalances'], len(input_parameters['cgas'][0, :])))
 
-    gas_indices = []
-    for i in gas_names:
-        gas_indices.append(names.index(i))
+    input_parameters['gas_indices'] = []
+    for i in input_parameters['gas_names']:
+        input_parameters['gas_indices'].append(input_parameters['names'].index(i))
 
-    if impose_Ptotal:
-        for i in range(Nbalances):
-            for j in range(len(Ptotal)):
-                for k in range(len(balances[i])):
-                    init_amounts[i][j] = init_amounts[i][j] + x0[balances[i][k]]
-        for i in range(Nbalances, Nbalances + Ngas_species):
-            for j in range(len(Ptotal)):
+    if input_parameters['impose_ptotal']:
+        for i in range(input_parameters['Nbalances']):
+            for j in range(len(input_parameters['Ptotal'])):
+                for k in range(len(input_parameters['balances'][i])):
+                    init_amounts[i][j] = init_amounts[i][j] + input_parameters['x0'][input_parameters['balances'][i][k]]
+        for i in range(input_parameters['Nbalances'], input_parameters['Nbalances'] + input_parameters['Ngas']):
+            for j in range(len(input_parameters['Ptotal'])):
                 init_amounts[i][j] = (
-                    (Ptotal[j] * gas_composition[i - Nbalances])
-                    * V
-                    * np.exp(-muex_gas[i - Nbalances] / (R * T))
-                    / (R * T)
+                    (input_parameters['Ptotal'][j] * input_parameters['gas_composition'][i - input_parameters['Nbalances']])
+                    * input_parameters['V']
+                    * np.exp(-input_parameters['muex_gas'][i - input_parameters['Nbalances']] / (R * input_parameters['T']))
+                    / (R * input_parameters['T'])
                 )
 
     else:
-        for i in range(Nbalances):
-            for j in range(len(cgas[0, :])):
-                for k in range(len(balances[i])):
-                    if balances[i][k] in gas_indices:
-                        ind = gas_names.index(names[balances[i][k]])
-                        init_amounts[i][j] = init_amounts[i][j] + cgas[ind, j]
+        for i in range(input_parameters['Nbalances']):
+            for j in range(len(input_parameters['cgas'][0, :])):
+                for k in range(len(input_parameters['balances'][i])):
+                    if input_parameters['balances'][i][k] in input_parameters['gas_indices']:
+                        ind = input_parameters['gas_names'].index(input_parameters['names'][input_parameters['balances'][i][k]])
+                        input_parameters['init_amounts'][i][j] = input_parameters['init_amounts'][i][j] + input_parameters['cgas'][ind, j]
                     else:
-                        init_amounts[i][j] = init_amounts[i][j] + x0[balances[i][k]]
+                        input_parameters['init_amounts'][i][j] = input_parameters['init_amounts'][i][j] + input_parameters['x0'][input_parameters['balances'][i][k]]
 
     # The function calc_K0 computes the desired equilibrium constants of the reactions in the system
-    K0 = calc_k0(
-        T,
-        lnK,
-        x0,
-        pure_density,
-        stoichiometry,
-        Nreactions,
-        mu_0,
-        mu_ex,
-        V,
-        solvent_index,
-        compute_Kdes,
-    )  # compute desired equilibrium constants
+    K0 = calc_k0(**input_parameters) # compute desired equilibrium constants
     lnK0 = np.log(K0)
     # The function calc_speciation computes the speciation as a function of gas concentration
-    if impose_Ptotal:
-        spec = calc_speciation_impose_ptotal(
-            T,
-            x0,
-            K0,
-            Nbalances,
-            balances,
-            init_amounts,
-            Nreactions,
-            stoichiometry,
-            cgas,
-            gas_indices,
-            gas_names,
-            charges,
-            solvent_index,
-            compute_Kdes,
-            lnK,
-            pure_density,
-            mu_0,
-            mu_ex,
-            V,
-            names,
-            Ptotal,
-        )
+    if input_parameters['impose_ptotal']:
+        spec = calc_speciation_impose_ptotal(**input_parameters)
     else:
-        spec = calc_speciation(
-            T,
-            x0,
-            K0,
-            Nbalances,
-            balances,
-            init_amounts,
-            Nreactions,
-            stoichiometry,
-            cgas,
-            gas_names,
-            charges,
-            solvent_index,
-            compute_Kdes,
-            lnK,
-            pure_density,
-            mu_0,
-            mu_ex,
-            V,
-            names,
-        )
+        spec = calc_speciation(**input_parameters)
 
     # This part prints the computed gas pressure as a function of gas loading
     # if impose_Ptotal:
@@ -190,7 +113,7 @@ def read_input(file_name):
     input_file = open(file_name, 'r', encoding='UTF-8')
     input_data = input_file.readlines()
     input_file.close()
-
+    input_parameters = dict()
     # Define Avogadro's constant
     NA = 6.022140857e23
 
@@ -201,167 +124,141 @@ def read_input(file_name):
     for line in input_data:
         if "Temperature" in line:
             index = input_data.index(line)
-            T = float(input_data[index + 1])  # Temperature
+            input_parameters['T'] = float(input_data[index + 1])
         if "Number of Species" in line:
             index = input_data.index(line)
             Nspecies = int(input_data[index + 1])  # Number of species
         if "C0" in line:
             index = input_data.index(line)
-            x0 = np.zeros(Nspecies)  # The initial guess in mol/dm3
+            input_parameters['x0'] = np.zeros(Nspecies)
             for i in range(Nspecies):
                 if float(input_data[index + 1].split()[i]) <= 0:
-                    x0[i] = 1e-10
+                    input_parameters['x0'][i] = 1e-10
                 else:
-                    x0[i] = float(input_data[index + 1].split()[i])
+                    input_parameters['x0'][i] = float(input_data[index + 1].split()[i])
         if "Names of species" in line:
             index = input_data.index(line)
-            names = []  # Names of the species
+            # Names of the species
+            input_parameters['names'] = []
             for i in range(Nspecies):
-                names.append(input_data[index + 1].split()[i])
+                input_parameters['names'].append(input_data[index + 1].split()[i])
         if "Charges" in line:
             index = input_data.index(line)
             # Net charges of the species, follows the indices in the the list of names
-            charges = np.zeros(Nspecies)
+            input_parameters['charges'] = np.zeros(Nspecies)
             for i in range(Nspecies):
-                charges[i] = int(input_data[index + 1].split()[i])
+                input_parameters['charges'][i] = int(input_data[index + 1].split()[i])
         if "mu^0 species" in line:
             index = input_data.index(line)
             # The ideal gas standard chemical potentials of the species in J/mol
-            mu_0 = np.zeros(Nspecies)
+            input_parameters['mu_0'] = np.zeros(Nspecies)
             for i in range(Nspecies):
                 # convert kJ/mol to J/mol and save the value
-                mu_0[i] = float(input_data[index + 1].split()[i]) * 1e3
+                input_parameters['mu_0'][i] = float(input_data[index + 1].split()[i]) * 1e3
         if "mu^ex species" in line:
             index = input_data.index(line)
             # The excess chemical potentials of the species in J/mol
-            mu_ex = np.zeros(Nspecies)
+            input_parameters['mu_ex'] = np.zeros(Nspecies)
             for i in range(Nspecies):
                 # convert kJ/mol to J/mol and save the value
-                mu_ex[i] = float(input_data[index + 1].split()[i]) * 1e3
+                input_parameters['mu_ex'][i] = float(input_data[index + 1].split()[i]) * 1e3
         if "Impose Ptotal and gas composition?" in line:
             index = input_data.index(line)
             true_answers = ["true", "t"]
-            impose_ptotal= input_data[index + 1].split()[0].lower() in true_answers
+            input_parameters['impose_ptotal'] = input_data[index + 1].split()[0].lower() in true_answers
             # if input_data[index + 1].split()[0].lower() in true_answers:
             #     impose_ptotal = True
             # else:
             #     impose_ptotal = False
         if "Ptotal / [kPa]" in line:
             index = input_data.index(line)
-            Ptotal = np.zeros(len(input_data[index + 1].split()))
-            for i in range(len(input_data[index + 1].split())):
-                Ptotal[i] = float(input_data[index + 1].split()[i]) * 1e3
+            input_parameters['Ptotal'] = float(input_data[index + 1].split()[i]) * 1e3
         if "Gas phase species" in line:
             index = input_data.index(line)
             Ngas = len(
                 input_data[index + 1].split()
-            )  # Number of species in the gas phase
-            gas_names = []  # Names of the species in the gas phase
+            )  
+            # Number of species in the gas phase
+            input_parameters['Ngas'] = len(input_data[index + 1].split())  
+            # Names of the species in the gas phase
+            input_parameters['gas_names'] = []
             for i in range(Ngas):
-                gas_names.append(input_data[index + 1].split()[i])
+                input_parameters['gas_names'].append(input_data[index + 1].split()[i])
         if "Gas phase composition" in line:
             index = input_data.index(line)
-            gas_composition = np.zeros(Ngas)
+            input_parameters['gas_composition'] = np.zeros(Ngas)
             for i in range(Ngas):
-                gas_composition[i] = float(input_data[index + 1].split()[i])
+                input_parameters['gas_composition'][i] = float(input_data[index + 1].split()[i])
             # Normalize gas phase composition so the total mole fraction is 1.
-            sum_gas_molefraction = sum(gas_composition)
-            gas_composition = np.divide(gas_composition, sum_gas_molefraction)
+            sum_gas_molefraction = sum(input_parameters['gas_composition'])
+            input_parameters['gas_composition'] = np.divide(input_parameters['gas_composition'], sum_gas_molefraction)
         if "mu^ex gases" in line:
             index = input_data.index(line)
             # The excess chemical potentials of the gas species in solvent at T in J/mol.
-            muex_gas = np.zeros(Ngas)
+            input_parameters['muex_gas'] = np.zeros(Ngas)
             for i in range(Ngas):
-                muex_gas[i] = (
-                    float(input_data[index + 1].split()[i]) * 1e3
-                )  # convert kJ/mol to J/mol
+                # convert kJ/mol to J/mol
+                input_parameters['muex_gas'][i] = float(input_data[index + 1].split()[i]) * 1e3
         if "Ctotal,gas" in line:
             index = input_data.index(line)
             # The concentrations of the gas species. This is a list.
-            cgas = np.zeros((Ngas, len(input_data[index + 1].split())))
+            input_parameters['cgas'] = np.zeros((Ngas, len(input_data[index + 1].split())))
             # The main loop of the solver uses this list.
             # This means the speciation is computed as a function of
             # the concentration of the gas species in the solution.
             for i in range(Ngas):
-                for j in range(len(cgas[i, :])):
-                    cgas[i, j] = float(input_data[index + i + 1].split()[j])
+                for j in range(len(input_parameters['cgas'][i, :])):
+                    input_parameters['cgas'][i, j] = float(input_data[index + i + 1].split()[j])
         if "Number of Reactions" in line:
             index = input_data.index(line)
-            Nreactions = int(input_data[index + 1])  # Number of reactions
+            # Number of reactions
+            input_parameters['Nreactions'] = int(input_data[index + 1])  
         if "Stoichiometry" in line:
             index = input_data.index(line)
-            stoichiometry = np.zeros(
-                (Nreactions, Nspecies)
-            )  # The matrix that contains the stoichiometry.
-            for i in range(Nreactions):
+            # The matrix that contains the stoichiometry.
+            input_parameters['stoichiometry'] = np.zeros((input_parameters['Nreactions'],Nspecies))
+            for i in range(input_parameters['Nreactions']):
                 for j in range(Nspecies):
-                    stoichiometry[i, j] = int(input_data[index + 1 + i].split()[j])
+                    input_parameters['stoichiometry'][i, j] = int(input_data[index + 1 + i].split()[j])
         if "Number of mass balance" in line:
             index = input_data.index(line)
-            Nbalances = int(
-                input_data[index + 1]
-            )  # Number of mass balances in the system
+            # Number of mass balances in the system
+            input_parameters['Nbalances'] = int(input_data[index + 1])
         if "Balances" in line:
             index = input_data.index(line)
             # This list contains lists of the species included in each balance equation.
-            balances = [
+            input_parameters['balances'] = [
                 []
-            ] * Nbalances
-            for i in range(Nbalances):
-                balances[i] = []
+            ] * input_parameters['Nbalances']
+            for i in range(input_parameters['Nbalances']):
+                input_parameters['balances'][i] = []
                 for j in range(len(input_data[index + 1 + i].split())):
-                    balances[i].append(int(input_data[index + 1 + i].split()[j]) - 1)
+                    input_parameters['balances'][i].append(int(input_data[index + 1 + i].split()[j]) - 1)
         if "ln(K)" in line:
             index = input_data.index(line)
-            lnK = np.zeros(
-                Nreactions
-            )  # this is list contains the ln(K) of each reaction.
-            compute_Kdes = np.full(
-                Nreactions, False
-            )  # Create a list of booleans for all reactions (compute_Kdes)
+            # this is list contains the ln(K) of each reaction.
+            input_parameters['lnK'] = np.zeros(input_parameters['Nreactions'])
+            input_parameters['compute_Kdes'] = np.full(input_parameters['Nreactions'],False)
+            # Create a list of booleans for all reactions (compute_Kdes)
             # If the boolean is True, then compute Kdes for that reaction using
             # the derived expression (in calc_k0())
             # for the molefraction-based equilibrium constants
             # compute_K_des[i] is true if the input is:
             key = "QMMC"
-            for i in range(Nreactions):
+            for i in range(input_parameters['Nreactions']):
                 if input_data[index + 1].split()[i] == key:
-                    lnK[i] = int(0.0)
-                    compute_Kdes[i] = True
+                    input_parameters['lnK'][i] = int(0.0)
+                    input_parameters['compute_Kdes'][i] = True
                 else:
-                    lnK[i] = float(input_data[index + 1].split()[i])
+                    input_parameters['lnK'][i] = float(input_data[index + 1].split()[i])
         if "Name of the solvent" in line:
             index = input_data.index(line)
-            solvent_index = names.index(input_data[index + 1].split("\n")[0])
+            input_parameters['solvent_index'] = input_parameters['names'].index(input_data[index + 1].split("\n")[0])
         if "Pure Density" in line:
             index = input_data.index(line)
-            pure_density = (
-                float(input_data[index + 1]) * NA / 1e27
-            )  # convert from mol/dm3 to molecules per cubic angstrom
-    return (
-        T,
-        Nspecies,
-        x0,
-        names,
-        charges,
-        mu_0,
-        mu_ex,
-        gas_names,
-        muex_gas,
-        cgas,
-        Nreactions,
-        stoichiometry,
-        Nbalances,
-        balances,
-        lnK,
-        compute_Kdes,
-        solvent_index,
-        pure_density,
-        Ngas,
-        impose_ptotal,
-        Ptotal,
-        gas_composition,
-    )
+            # convert from mol/dm3 to molecules per cubic angstrom
+            input_parameters['pure_density'] = float(input_data[index + 1]) * NA / 1e27
+    return input_parameters
 
 
 def compute_mole_fractions(x):
@@ -1010,8 +907,9 @@ def write_output(
             + "Othonas A. Moultos and Thijs J. H. Vlugt \n"
             + "Please cite:  H.M. Polat, F. de Meyer, C. Houriez,"
             + " O.A. Moultos and T.J.H. Vlugt, 2023, \n"
-            + "Solving Chemical Reaction Equilibrium Using Free Energy and "
-            + "Quantum Chemistry Calculations: Modeling and Limitations,\n"
+            + "Solving Chemical Absorption Equilibria Using Free Energy and"
+            + "Quantum Chemistry Calculations: Methodology, Limitations, "
+            + "and New Open-Source Software,\n"
             + "Journal of Chemical Theory and Computation.\n\n"
             + "--------------------------------------------"
             + "Initial conditions--------------------------------------------\n"
